unit MainForm;

interface

uses NSys, NCommand, NConf, NCanvas, NForm, Element, SnapForm;

var
  Command: TCommand;
  Config: TConf;
  ProfDir: string;

type
  TNkMonMainForm = class(TSnapForm)
    AlphaBits: Pointer;
    Elements: array of TElement;
    MemCanvas: TMemCanvas;
    Timer: TTimer;
    constructor Create; override;
    procedure Closing; override;
    procedure DispatchCommand(Id: Longint); override;
    procedure LoadLayout;
    procedure UpdateElements;
    procedure MouseDown(const Button: TMouseButton; CX, CY: Longint); override;
    procedure ShowContextMenu(SX, SY: Longint); override;
  published
    procedure cmAbout;
    procedure cmExit;
    procedure cmReloadLayout;
    procedure cmStayOnTop;
  end;

implementation

uses Windows, NLib, NSock, Monitor, Version, Menu, Default;

constructor TNkMonMainForm.Create;
begin
  inherited;
  SetWindowLong(Handle, GWL_EXSTYLE,
    GetWindowLong(Handle, GWL_EXSTYLE) or WS_EX_LAYERED);
  SetBorder(False);
  SetToolWindow(True);
  LoadFormPos(Self, Config);

  LoadLayout;
  MemCanvas := TMemCanvas.Create(Handle, 32);
  MemCanvas.BeginDraw;
  UpdateElements;
  Timer := TTimer.Create(Self, UpdateElements, 1000);
  Timer.Start;
  Show;
end;

procedure TNkMonMainForm.Closing;
var
  position: TConf;
begin
  position := TConf.Create;
  try
    SaveFormPos(Self, position);
    try
      position.SaveToFile(JoinPath([ProfDir, 'position']));
    except
      on E: EIOError do App.Error(E.Path+#10#10+E.Msg);
    end;
  finally
    position.Free;
  end;
end;

procedure TNkMonMainForm.LoadLayout;
var
  cnt, i: Integer;
  line, path: string;
  s: TStream;
begin
  Width := 48;
  BgAlpha := 96;
  SetLength(Elements, 10);
  path := JoinPath([ProfDir, 'layout']);
  try
    if IsFile(path) then
      s := TFile.Open(path)
    else
      s := TBytesStream.Create(Layout, True);
  except
    on E: EIOError do
    begin
      Warn(FormatException(E));
      s := TBytesStream.Create(Layout, True);
    end;
  end;
  cnt := 0;
  while TConf.ReadLine(s, line) do
  begin
    if cnt = Length(Elements) then
      SetLength(Elements, cnt + cnt div 4);
    Elements[cnt] := CreateElement(SplitCommandLine(line));
    if Elements[cnt] <> nil then
      Inc(cnt);
  end;
  s.Free;
  SetLength(Elements, cnt);

  Monitor.Init;
  for i := 0 to High(Elements) do
    Elements[i].Init;
end;

procedure TNkMonMainForm.UpdateElements;
var
  i, prevwidth: Integer;
  pt: TPoint;
  size: TSize;
  bf: TBlendFunction;
  c, ec, a: PLongword;
begin
  // alpha: transparent 0<->255 opaque

  prevwidth := Width;
  try
    Monitor.Update;
  except
    on E: ENSockError do Warn(SysError(E.ErrNo));
  end;
  for i := 0 to High(Elements) do
    Elements[i].Update;

  // get alpha
  if AlphaBits <> nil then
  begin
    Current.Y := 0;
    MemCanvas.FillRect(0, 0, prevwidth, Height, BgAlpha);
    for i := 0 to High(Elements) do
      Elements[i].DrawAlpha(MemCanvas);
    Move(MemCanvas.Bits^, AlphaBits^, prevwidth*Height*4);
  end;

  // draw
  Current.Y := 0;
  MemCanvas.FillRect(0, 0, prevwidth, Height, 0);
  for i := 0 to High(Elements) do
    Elements[i].Draw(MemCanvas);

  // set alpha
  if AlphaBits <> nil then
  begin
    c := MemCanvas.Bits;
    ec := @PChar(c)[prevwidth*Height*4];
    a := AlphaBits;
    while PChar(c) < PChar(ec) do
    begin
      c^ := c^ or (a^ shl 24);
      {
      // premultiplied
      PChar(c)[0] := Char(Byte(PChar(c)[0]) * a^ div 255);
      PChar(c)[1] := Char(Byte(PChar(c)[1]) * a^ div 255);
      PChar(c)[2] := Char(Byte(PChar(c)[2]) * a^ div 255);
      }
      Inc(c);
      Inc(a);
    end;
  end;

  pt.x := 0;
  pt.y := 0;
  size.cx := prevwidth;
  size.cy := Height;
  bf.BlendOp := AC_SRC_OVER;
  bf.BlendFlags := 0;
  bf.SourceConstantAlpha := 255;
  bf.AlphaFormat := AC_SRC_ALPHA;
  UpdateLayeredWindow(
    Handle, 0, nil, @size, MemCanvas.Handle, @pt, 0, @bf, ULW_ALPHA);

  if (Current.Y <> Height) or (prevwidth <> Width) then
  begin
    Height := Current.Y;
    SetSize(Width, Height);
    MemCanvas.SetSize(Width, Height);
    ReallocMem(AlphaBits, Width*Height*4);
  end;
end;

procedure TNkMonMainForm.MouseDown(const Button: TMouseButton; CX, CY: Longint);
begin
  StartDrag;
end;

procedure TNkMonMainForm.ShowContextMenu(SX, SY: Longint);
begin
  with TPopupMenu.Create do
    try
      Command.AddMenuItems(Handle, MenuItems);
      CheckItem(Command.GetId(cmStayOnTop), StayOnTop);
      Popup(Self.Handle, SX, SY);
    finally
      Free;
    end;
end;

procedure TNkMonMainForm.DispatchCommand(Id: Longint);
begin
  Command.DispatchCommand(Id);
end;

{ Command Methods }

procedure TNkMonMainForm.cmReloadLayout;
begin
  LoadLayout;
end;

procedure TNkMonMainForm.cmStayOnTop;
begin
  SetStayOnTop(not StayOnTop);
end;

procedure TNkMonMainForm.cmAbout;
begin
  App.About(
    App.Name+' '+Version_+' ('+ReleaseDate+')'#10+
    #10+
    Copyright+#10+
    'This software is provided under the zlib/libpng license.'#10+
    #10+
    'Profile Directory: '+ProfDir);
end;

procedure TNkMonMainForm.cmExit;
begin
  Close;
end;

end.
